

UNIT Z80Load;

(***************************************************************************)
(*                                                                         *)
(*  Professional Adventure Writing System extractor.                       *)
(*  Author: Jose Luis Cebrin Page                                        *)
(*  Pascal version : Carlos Sanchez (csanchez@temple.subred.org)           *)
(*                                                                         *)
(***************************************************************************)
(*                                                                         *)
(*  Z80 files support and other minor changes added by Carlos Sanchez      *)
(*  when translating the code from C to pascal                             *)
(*                                                                         *)
(***************************************************************************)

INTERFACE

FUNCTION LoadZ80(FileName: String; VAR Z80RAM):Integer;
(*
Loads a .Z80 file in a memory zone. The memory zone is supposed to be
the beginning of Spectrum's RAM, at address 4000h (16384)

This function supports files from Z80 2.01 or later, to use older files
just load then inside Z80 emulator and save it again or use any format
changer to make it a SNA or SP file
*)

IMPLEMENTATION
{$X+}



TYPE PByte=^Byte;

FUNCTION GetByte(VAR P:PByte; VAR Counter:Word):Byte;
begin
 GetByte:=P^;
 Inc(longint(P));
 Dec(COunter);
END;

PROCEDURE Decode(VAR SRC,DST; Offset,BlockLen:Word);
VAR P0,PF:PByte;
    A,B:Byte;
BEGIN
 P0:=@Src;
 PF:=Pointer(Longint(@Dst)+Offset);
 REPEAT
  A:=GetByte(P0,BlockLen);
  IF A<>237 THEN BEGIN
                  PF^:=A;
                  Inc(Pf)
                 END
            ELSE BEGIN
                  B:=GetByte(P0,BlockLen);
                  IF B<>237 THEN BEGIN
                                  PF^:=237;
                                  Inc(Pf);
                                  PF^:=B;
                                  Inc(Pf);
                                 END
                            ELSE BEGIN
                                  A:=GetByte(P0,BlockLen);
                                  B:=GetByte(P0,BlockLen);
                                  FillChar(PF^,A,B);
                                  Inc(PF,A);
                                 END;
                 END;
 UNTIL BlockLen=0;
END;

FUNCTION LoadZ80(FileName: String; VAR Z80RAM):Integer;
VAR F:FILE;
    P:Pointer;
    AddHeaderSize:Word;
    BlockLen: Word;
    Page: Byte;
    HWMode:Byte;
    Offset:Word;
BEGIN
 LoadZ80:=0; (* no hay error *)
 Assign(F,FileName);
 Reset(F,1);
 Seek(F,30);
 BlockRead(F,AddHeaderSize,2);
 IF (AddHeaderSize<>23) AND (AddHeaderSize<>54) THEN BEGIN
                                         LoadZ80:=1; (* Invalid file *)
                                         Exit
                                        END;
 Seek(F,34);
 BlockRead(F,HWMode,1);
 IF ((AddHeaderSize=23) AND (HWMode>2)) OR
    ((AddHeaderSize=54) AND (HWMode>3)) THEN BEGIN
                                              LoadZ80:=2; (* No a 48K file*)
                                              Exit
                                             END;
 Seek(F,32+AddHeaderSize);
 WHILE NOT EOF(F) DO
  BEGIN
   BlockRead(F,BlockLen,2);
   BlockRead(F,Page,1);
   GetMem(P,BlockLen);
   BlockRead(F,P^,BlockLen);
   CASE Page OF
         4:Offset:=32768;
         5:Offset:=49152;
         8:Offset:=16384;
         6,7:BEGIN
              Offset:=0;
             END
         ELSE BEGIN
               LoadZ80:=1;
               Exit
              END;
   END;
   IF Offset<>0 THEN  BEGIN
                       Dec(Offset,16384);
                       Decode(P^,Z80RAM,Offset,BlockLen);
                      END;
 END;
Close(F);
END;

END.